@@ -495,6 +495,36 @@ Eigen::VectorXd lsum_col(
 }
 
 
+py::array_t<double> lsum_row_py(
+    py::array_t<double> x_values,
+    bool is_log = true
+){
+    return py::cast(
+        lsum_row(
+            x_values.cast<Eigen::Matrix<
+        double,
+        Eigen::Dynamic,
+        Eigen::Dynamic>>(),
+            is_log
+        )
+    );
+}
+py::array_t<double> lsum_col_py(
+    py::array_t<double> x_values,
+    bool is_log = true
+){
+    return py::cast(
+        lsum_col(
+            x_values.cast<Eigen::Matrix<
+        double,
+        Eigen::Dynamic,
+        Eigen::Dynamic>>(),
+            is_log
+        )
+    );
+}
+
+
 py::array_t<double> lsum_py(
     py::array_t<double> x_values,
     bool is_log = true,
@@ -821,6 +851,20 @@ PYBIND11_MODULE(c_archimedean, m) {
     )pbdoc",
     py::arg("u_values"), py::arg("theta"), py::arg("is_log"));
 
+    m.def("lsum_row", &lsum_row_py, R"pbdoc(
+        :param x_values: np.array
+        :param is_log: bool
+        :return:
+    )pbdoc",
+    py::arg("x_values"), py::arg("is_log"));
+
+    m.def("lsum_col", &lsum_col_py, R"pbdoc(
+        :param x_values: np.array
+        :param is_log: bool
+        :return:
+    )pbdoc",
+    py::arg("x_values"), py::arg("is_log"));
+
     m.def("lsum", &lsum_py, R"pbdoc(
         :param x_values: np.array
         :param axis: int
