@@ -25,6 +25,46 @@ from mpmath import polylog
 import c_archimedean as c_arch
 
 
+def explsum(x_value, is_log=False, axis=0):
+    """
+    compute log sum_i x_i
+    :param x_values:
+    :param is_log:
+    :param axis:
+    :return:
+    >>> explsum(np.array([1, 2, 3, 4]), is_log=False)
+    array([10.])
+    >>> explsum(np.array([0, 1, 2, 3, 4]), axis=0)
+    array([10.])
+    >>> explsum(np.array([[0.42873569, 0.18285458, 0.9514195],
+    ...    [0.25148149, 0.05617784, 0.3378213],
+    ...    [0.79410993, 0.76175687, 0.0709562],
+    ...    [0.02694249, 0.45788802, 0.6299574],
+    ...    [0.39522060, 0.02189511, 0.6332237],
+    ...    [0.66878367, 0.38075101, 0.5185625],
+    ...    [0.90365653, 0.19654621, 0.6809525],
+    ...    [0.28607729, 0.82713755, 0.7686878],
+    ...    [0.22437343, 0.16907646, 0.5740400],
+    ...    [0.66752741, 0.69487362, 0.3329266]]),
+    ... axis=0
+    ... )
+    array([4.64690853, 3.74895727, 5.4985475 ])
+    >>> explsum(np.array([[0.42873569, 0.18285458, 0.9514195],
+    ...    [0.25148149, 0.05617784, 0.3378213],
+    ...    [0.79410993, 0.76175687, 0.0709562],
+    ...    [0.02694249, 0.45788802, 0.6299574],
+    ...    [0.39522060, 0.02189511, 0.6332237],
+    ...    [0.66878367, 0.38075101, 0.5185625],
+    ...    [0.90365653, 0.19654621, 0.6809525],
+    ...    [0.28607729, 0.82713755, 0.7686878],
+    ...    [0.22437343, 0.16907646, 0.5740400],
+    ...    [0.66752741, 0.69487362, 0.3329266]]),
+    ... axis=1)
+    array([1.56300977, 0.64548063, 1.626823  , 1.11478791, 1.05033941,
+           1.56809718, 1.78115524, 1.88190264, 0.96748989, 1.69532763])
+    """
+    return np.exp(lsum(x_value, is_log=is_log, axis=axis))
+
 def lsum(x_values, is_log=True, axis=0):
     """
     compute log sum_i x_i
@@ -32,8 +72,8 @@ def lsum(x_values, is_log=True, axis=0):
     :param is_log:
     :param axis:
     :return:
-    # >>> lsum(np.array([1, 2, 3, 4]))
-    # 4.440189698561196
+    >>> np.exp(lsum(np.array([1, 2, 3, 4]), is_log=False))
+    array([10.])
     >>> lsum(np.array([[0.42873569, 0.18285458, 0.9514195],
     ...    [0.25148149, 0.05617784, 0.3378213],
     ...    [0.79410993, 0.76175687, 0.0709562],
@@ -43,7 +83,8 @@ def lsum(x_values, is_log=True, axis=0):
     ...    [0.90365653, 0.19654621, 0.6809525],
     ...    [0.28607729, 0.82713755, 0.7686878],
     ...    [0.22437343, 0.16907646, 0.5740400],
-    ...    [0.66752741, 0.69487362, 0.3329266]])
+    ...    [0.66752741, 0.69487362, 0.3329266]]),
+    ... axis=0
     ... )
     array([2.8032607 , 2.71855056, 2.87969188])
     >>> lsum(np.array([[0.42873569, 0.18285458, 0.9514195],
@@ -60,7 +101,9 @@ def lsum(x_values, is_log=True, axis=0):
     array([1.67247612, 1.32058311, 1.69157834, 1.50086587, 1.479448  ,
            1.62823149, 1.73370009, 1.75351973, 1.43774153, 1.67670621])
     """
-    return c_arch.lsum(x_values, axis, is_log)
+    if axis == 0:
+        return c_arch.lsum_row(x_values=x_values, is_log=is_log)
+    return c_arch.lsum_col(x_values=x_values, is_log=is_log)
 
 
 def lssum(x_values, x_sign=np.nan, is_log=True):
@@ -1272,3 +1315,7 @@ def pdf_gumbel(u_values, theta, is_log=False):
            -0.4483621 , -5.15135971, -2.95009109,  0.94790989,  1.04000301])
     """
     return c_arch.pdf_gumbel(u_values, theta, is_log)
+
+if __name__ == "__main__":
+    import doctest
+    doctest.testmod()
