@@ -749,12 +749,14 @@ def merge_beds(bed_files: list,
 
 
 def narrowpeaks2array(np_list: list,
-                      score_cols: str = None) -> np.array:
+                      score_cols: str = None,
+                      outdir: str = None) -> np.array:
     """
     convert a list of pd.DataFrame representing bed files to an np.array of
     their score column
     :type np_list: list[pd.DataFrame]
     :type score_cols: str colname of the score column
+    :type outdir: str output directory
     :param np_list: list of pd.DataFrame representing bed files
     :return np.array whose columns are the score columns of the bed files
 
@@ -775,6 +777,27 @@ def narrowpeaks2array(np_list: list,
     return scores
 
 
+def nparray2tsv(np_array: np.array,
+                outdir: str = None,
+                filename: str = "matrix") -> np.array:
+    """
+    If outdir is set write a matrix.tsc file in the outdir folder
+    :param np_array:
+    :param outdir:
+    :param filename:
+    :return:
+    """
+    if outdir != None:
+        output_name = PurePath(outdir).joinpath(filename + ".tsv")
+        pd.DataFrame(np_array).to_csv(
+            output_name, sep='\t',
+            encoding='utf-8',
+            header=False,
+            index=False
+        )
+    return np_array
+
+
 def process_bed(file_names: list,
                 outdir: str,
                 idr_func: Callable[[np.array], np.array],
@@ -819,9 +842,12 @@ def process_bed(file_names: list,
     )
     if bed_files[0].shape[0] > 0:
         local_idr = idr_func(
-            narrowpeaks2array(
-                np_list=bed_files[1:],
-                score_cols=score_cols
+            nparray2tsv(
+                narrowpeaks2array(
+                    np_list=bed_files[1:],
+                    score_cols=score_cols,
+                ),
+                outdir=outdir
             ),
             threshold=threshold,
             missing=missing
