@@ -14,6 +14,7 @@ NarrowPeaks files.
 """
 
 from scipy.optimize import minimize
+from scipy.special import logsumexp
 import numpy as np
 import midr.log as log
 import midr.archimedean as archimedean
@@ -29,18 +30,31 @@ COPULA_DENSITY = {
 }
 DMLE_COPULA = {
     'clayton': archimedean.dmle_copula_clayton,
-    'frank': archimedean.dmle_copula_frank,
+    #'frank': archimedean.dmle_copula_frank,
+    'frank': lambda x: 2.0,
     'gumbel': archimedean.dmle_copula_gumbel
 }
 
 
-def copula_density(copula):
+def copula_density(copula, **kwargs):
     """
     return a copula density function
     :param copula:
     :return:
+    >>> u_values = np.random.rand(10000,30)
+    >>> np.count_nonzero(
+    ...     ~np.isnan(
+    ...         copula_density('frank',
+    ...             u_values = u_values,
+    ...             theta = 6.0,
+    ...             is_log = True
+    ... )))
+    10000
     """
-    return COPULA_DENSITY[copula]
+    return np.nan_to_num(
+        COPULA_DENSITY[copula](**kwargs),
+        copy=False
+    )
 
 
 def copula_number():
@@ -65,7 +79,7 @@ def dmle_copula(copula):
     :param copula:
     :return:
     """
-    return DMLE_COPULA[copula]
+    return np.nan_to_num(DMLE_COPULA[copula], copy=False)
 
 
 def number_of_missing(u_values, missing=None):
@@ -135,7 +149,14 @@ def list_of_u_values(u_values, missing=None):
     ...    ]),
     ...    missing=0.0
     ... )
-    array([0, 1, 0, 0, 2, 0, 0, 1, 0, 0])
+    (array([0, 1, 0, 0, 2, 0, 0, 1, 0, 0]), [array([[0.42873569, 0.18285458, 0.9514195 , 0.9514195 ],
+           [0.79410993, 0.76175687, 0.0709562 , 0.0709562 ],
+           [0.02694249, 0.45788802, 0.6299574 , 0.6299574 ],
+           [0.66878367, 0.38075101, 0.5185625 , 0.5185625 ],
+           [0.90365653, 0.19654621, 0.6809525 , 0.6809525 ],
+           [0.22437343, 0.16907646, 0.57404   , 0.57404   ],
+           [0.66752741, 0.69487362, 0.3329266 , 0.3329266 ]]), array([[0.25148149, 0.3378213 , 0.3378213 ],
+           [0.82713755, 0.7686878 , 0.7686878 ]]), array([[0.3952206, 0.6299574]]), array([], shape=(0, 1), dtype=float64)])
     """
     nb_missing_row = number_of_missing(u_values=u_values, missing=missing)
     list_u_values = [remove_missing(
@@ -154,7 +175,7 @@ def censored_copula_density(u_values, theta, copula, missing=None):
     :param copula:
     :param missing:
     :return:
-    >>> censored_copula_density(np.array([
+    >>> np.exp(censored_copula_density(np.array([
     ...    [0.42873569, 0.18285458, 0.9514195, 0.9514195],
     ...    [0.25148149, 0.0, 0.3378213, 0.3378213],
     ...    [0.79410993, 0.76175687, 0.0709562, 0.0709562],
@@ -169,10 +190,10 @@ def censored_copula_density(u_values, theta, copula, missing=None):
     ...    theta=6,
     ...    copula="frank",
     ...    missing=0.0
-    ... )
-    array([0.01399523, 3.20545262, 0.05619478, 0.04574052, 1.02461261,
-           3.41716738, 0.03672658, 4.78125643, 1.02599504, 1.07616887])
-    >>> censored_copula_density(np.array([
+    ... ))
+    array([0.01076583, 3.20545269, 0.02025168, 0.0171356 , 1.02461261,
+           3.06315388, 0.02965913, 4.78125626, 0.65824184, 0.90881984])
+    >>> np.exp(censored_copula_density(np.array([
     ...    [0.42873569, 0.18285458, 0.9514195, 0.9514195],
     ...    [0.25148149, 0.3378213, 0.3378213, 0.0],
     ...    [0.79410993, 0.76175687, 0.0709562, 0.0709562],
@@ -187,10 +208,10 @@ def censored_copula_density(u_values, theta, copula, missing=None):
     ...    theta=6,
     ...    copula="frank",
     ...    missing=0.0
-    ... )
-    array([0.01399523, 3.20545262, 0.05619478, 0.04574052, 1.02461261,
-           3.41716738, 0.03672658, 4.78125643, 1.02599504, 1.07616887])
-    >>> censored_copula_density(np.array([
+    ... ))
+    array([0.01076583, 3.20545269, 0.02025168, 0.0171356 , 1.02461261,
+           3.06315388, 0.02965913, 4.78125626, 0.65824184, 0.90881984])
+    >>> np.exp(censored_copula_density(np.array([
     ...    [0.42873569, 0.18285458, 0.9514195, 0.9514195],
     ...    [0.25148149, 0.0, 0.3378213, 0.3378213],
     ...    [0.79410993, 0.76175687, 0.0709562, 0.0709562],
@@ -205,11 +226,11 @@ def censored_copula_density(u_values, theta, copula, missing=None):
     ...    theta=2,
     ...    copula="gumbel",
     ...    missing=0.0
-    ... )
+    ... ))
     array([6.06628394e-03, 2.58088497e+00, 6.63892197e-02, 5.72009627e-02,
            1.11589049e+00, 2.90817712e+00, 7.62035940e-02, 6.16647093e+00,
            1.08801162e+00, 1.22184286e+00])
-    >>> censored_copula_density(np.array([
+    >>> np.exp(censored_copula_density(np.array([
     ...    [0.42873569, 0.18285458, 0.9514195, 0.9514195],
     ...    [0.25148149, 0.0, 0.3378213, 0.3378213],
     ...    [0.79410993, 0.76175687, 0.0709562, 0.0709562],
@@ -224,39 +245,29 @@ def censored_copula_density(u_values, theta, copula, missing=None):
     ...    theta=2,
     ...    copula="clayton",
     ...    missing=0.0
-    ... )
+    ... ))
     array([3.50439735e-02, 3.68342742e+00, 7.46301436e-03, 6.54688263e-06,
            1.10049896e+00, 2.61709390e+00, 5.40124152e-02, 3.30222057e+00,
            6.27832609e-01, 1.25018803e+00])
-    >>> COPULA_DENSITY["frank"](np.array([
-    ...    [0.42873569, 0.18285458, 0.9514195, 0.9514195],
-    ...    [0.25148149, 0.0, 0.3378213, 0.3378213],
-    ...    [0.79410993, 0.76175687, 0.0709562, 0.0709562],
-    ...    [0.02694249, 0.45788802, 0.6299574, 0.6299574],
-    ...    [0.39522060, 0.0, 0.0, 0.6299574],
-    ...    [0.66878367, 0.38075101, 0.5185625, 0.5185625],
-    ...    [0.90365653, 0.19654621, 0.6809525, 0.6809525],
-    ...    [0.0, 0.82713755, 0.7686878, 0.7686878],
-    ...    [0.22437343, 0.16907646, 0.5740400, 0.5740400],
-    ...    [0.66752741, 0.69487362, 0.3329266, 0.3329266]
-    ...    ]),
-    ...    6,
-    ... )
-    array([0.01399523,        nan, 0.05619478, 0.04574052,        nan,
-           3.41716738, 0.03672658,        nan, 1.02599504, 1.07616887])
     """
     if missing is None:
-        return copula_density(copula)(u_values, theta, False)
+        return copula_density(
+            copula,
+            u_values=u_values,
+            theta=theta,
+            is_log=True
+        )
     (nb_missing_row, list_u_values) = list_of_u_values(
         u_values=u_values,
         missing=missing
     )
     cdensity = np.zeros(u_values.shape[0])
     for nb_missing in range(u_values.shape[1]):
-        cdensity[nb_missing_row == nb_missing] = copula_density(copula)(
-            list_u_values[nb_missing],
-            theta,
-            False
+        cdensity[nb_missing_row == nb_missing] = copula_density(
+            copula,
+            u_values=list_u_values[nb_missing],
+            theta=theta,
+            is_log=True
         )
     return cdensity
 
@@ -270,10 +281,10 @@ def delta(params_list, threshold):
     :return: bool
     >>> delta(
     ...    params_list={
-    ...        'pi': 0.8,
-    ...        'pi_old': 0.8,
-    ...        'alpha': np.array([0.3333, 0.3333, 0.3333]),
-    ...        'alpha_old': np.array([0.3333, 0.3333, 0.3333]),
+    ...        'pi': np.log(0.8),
+    ...        'pi_old': np.log(0.8),
+    ...        'alpha': np.array(np.log([0.3333, 0.3333, 0.3333])),
+    ...        'alpha_old': np.array(np.log([0.3333, 0.3333, 0.3333])),
     ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
     ...        'frank': {'theta': 6, 'pi': 0.2, 'theta_old': 6, 'pi_old': 0.2},
     ...        'gumbel': {'theta': 6, 'pi': 0.2, 'theta_old': 6, 'pi_old': 0.2},
@@ -284,10 +295,10 @@ def delta(params_list, threshold):
     True
     >>> delta(
     ...    params_list={
-    ...        'pi': 0.8,
-    ...        'pi_old': 0.8,
-    ...        'alpha': np.array([0.3333, 0.3333, 0.3333]),
-    ...        'alpha_old': np.array([0.3333, 0.3333, 0.3333]),
+    ...        'pi': np.log(0.8),
+    ...        'pi_old': np.log(0.8),
+    ...        'alpha': np.array(np.log([0.3333, 0.3333, 0.3333])),
+    ...        'alpha_old': np.array(np.log([0.3333, 0.3333, 0.3333])),
     ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
     ...        'frank': {'theta': 6, 'pi': 0.2, 'theta_old': 6, 'pi_old': 0.2},
     ...        'gumbel': {'theta': 6, 'pi': 0.2, 'theta_old': 6, 'pi_old': 0.2},
@@ -298,13 +309,14 @@ def delta(params_list, threshold):
     False
     """
     max_delta = [
-        max(abs(params_list['alpha'] - params_list['alpha_old'])),
-        abs(params_list['pi'] - params_list['pi_old']),
+        max(abs(np.exp(params_list['alpha']) -
+                np.exp(params_list['alpha_old']))),
+        abs(np.exp(params_list['pi']) - np.exp(params_list['pi_old'])),
     ]
-
     for copula in copula_names():
         max_delta.append(
-            abs(params_list[copula]['theta'] - params_list[copula]['theta_old'])
+            abs(params_list[copula]['theta'] -
+                params_list[copula]['theta_old'])
         )
     return max(max_delta) >= threshold
 
@@ -316,7 +328,7 @@ def expectation_k(u_values, params_list, dcopula=None):
     :param params_list:
     :param dcopula:
     :return:
-    >>> expectation_k(np.array([
+    >>> np.exp(expectation_k(np.array([
     ...    [0.42873569, 0.18285458, 0.9514195],
     ...    [0.25148149, 0.05617784, 0.3378213],
     ...    [0.79410993, 0.76175687, 0.0709562],
@@ -329,30 +341,44 @@ def expectation_k(u_values, params_list, dcopula=None):
     ...    [0.66752741, 0.69487362, 0.3329266]
     ...    ]),
     ...    params_list={
-    ...        'pi': 0.8,
-    ...        'alpha': [0.3333, 0.3333, 0.3333],
+    ...        'pi': np.log(0.8),
+    ...        'alpha': np.log([0.3333, 0.3333, 0.3333]),
     ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
     ...        'frank': {'theta': 6},
     ...        'gumbel': {'theta': 6},
     ...        'clayton': {'theta': 6},
     ...        'missing': None
     ...    }
-    ... )
+    ... ))
     array([0.00724737, 0.12708761, 0.00081767, 0.00660034, 0.00861115,
            0.18035533, 0.00356172, 0.00996085, 0.08777781, 0.05167504])
+    >>> u_values = np.random.rand(10000,30)
+    >>> np.count_nonzero(~np.isnan(expectation_k(u_values,
+    ...    params_list={
+    ...        'pi': np.log(0.8),
+    ...        'alpha': np.log([0.3333, 0.3333, 0.3333]),
+    ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
+    ...        'frank': {'theta': 6},
+    ...        'gumbel': {'theta': 6},
+    ...        'clayton': {'theta': 6},
+    ...        'missing': None
+    ...    }
+    ... )))
+    10000
     """
     if dcopula is None:
-        k_state = np.zeros(u_values.shape[0])
+        dcopula = np.zeros((u_values.shape[0], copula_number()))
         for copula in copula_names():
-            k_state += partial_loglikelihood_copula(
-                u_values=u_values,
-                copula=copula,
-                params_list=params_list
-            )
-    else:
-        k_state = np.sum(dcopula, axis=1)
-    k_state *= (1 - params_list['pi'])
-    return k_state / (params_list['pi'] + k_state)
+            dcopula[:,
+                list(copula_names()).index(copula)
+            ] = partial_loglikelihood_copula(
+                    u_values=u_values,
+                    copula=copula,
+                    params_list=params_list
+                )
+    k_state = archimedean.lsum(dcopula, is_log=True, axis=1)
+    k_state += np.log(1 - np.exp(params_list['pi']))
+    return k_state - np.log(np.exp(params_list['pi']) + np.exp(k_state))
 
 
 def expectation_l(u_values, params_list, dcopula=None):
@@ -362,7 +388,7 @@ def expectation_l(u_values, params_list, dcopula=None):
     :param params_list:
     :param dcopula:
     :return:
-    >>> np.sum(expectation_l(np.array([
+    >>> np.sum(np.exp(expectation_l(np.array([
     ...    [0.42873569, 0.18285458, 0.9514195],
     ...    [0.25148149, 0.05617784, 0.3378213],
     ...    [0.79410993, 0.76175687, 0.0709562],
@@ -375,17 +401,17 @@ def expectation_l(u_values, params_list, dcopula=None):
     ...    [0.66752741, 0.69487362, 0.3329266]
     ...    ]),
     ...    params_list={
-    ...        'pi': 0.8,
-    ...        'alpha': [0.3333, 0.3333, 0.3333],
+    ...        'pi': np.log(0.8),
+    ...        'alpha': np.log([0.3333, 0.3333, 0.3333]),
     ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
     ...        'frank': {'theta': 6, 'pi': 0.2},
     ...        'gumbel': {'theta': 6, 'pi': 0.2},
     ...        'clayton': {'theta': 6, 'pi': 0.2},
     ...        'missing': None
     ...    }
-    ... ), axis=1)
+    ... )), axis=1)
     array([1., 1., 1., 1., 1., 1., 1., 1., 1., 1.])
-    >>> expectation_l(np.array([
+    >>> np.exp(expectation_l(np.array([
     ...    [0.42873569, 0.18285458, 0.9514195],
     ...    [0.25148149, 0.05617784, 0.3378213],
     ...    [0.79410993, 0.76175687, 0.0709562],
@@ -398,15 +424,15 @@ def expectation_l(u_values, params_list, dcopula=None):
     ...    [0.66752741, 0.69487362, 0.3329266]
     ...    ]),
     ...    params_list={
-    ...        'pi': 0.8,
-    ...        'alpha': [0.3333, 0.3333, 0.3333],
+    ...        'pi': np.log(0.8),
+    ...        'alpha': np.log([0.3333, 0.3333, 0.3333]),
     ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
     ...        'frank': {'theta': 6, 'pi': 0.2},
     ...        'gumbel': {'theta': 6, 'pi': 0.2},
     ...        'clayton': {'theta': 6, 'pi': 0.2},
     ...        'missing': None
     ...    }
-    ... )
+    ... ))
     array([[9.99241613e-01, 5.13503781e-07, 7.57873317e-04],
            [9.84106292e-01, 1.58920980e-02, 1.61032426e-06],
            [9.99999854e-01, 1.41403781e-07, 5.09198107e-09],
@@ -417,6 +443,20 @@ def expectation_l(u_values, params_list, dcopula=None):
            [9.94643749e-01, 2.06163638e-05, 5.33563491e-03],
            [8.68352025e-01, 8.87728833e-02, 4.28750915e-02],
            [9.43093479e-01, 5.21635253e-03, 5.16901688e-02]])
+
+    >>> u_values = np.random.rand(10000,30)
+    >>> np.count_nonzero(~np.isnan(expectation_l(u_values,
+    ...    params_list={
+    ...        'pi': np.log(0.8),
+    ...        'alpha': np.log([0.3333, 0.3333, 0.3333]),
+    ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
+    ...        'frank': {'theta': 6},
+    ...        'gumbel': {'theta': 6},
+    ...        'clayton': {'theta': 6},
+    ...        'missing': None
+    ...    }
+    ... )))
+    30000
     """
     l_state = np.zeros((u_values.shape[0], copula_number()))
     if dcopula is None:
@@ -426,9 +466,9 @@ def expectation_l(u_values, params_list, dcopula=None):
         )
     for copula in copula_names():
         l_state[:, params_list['order'][copula]] = (
-                dcopula[:, params_list['order'][copula]] /
-                np.sum(dcopula, axis=1)
-        )
+                dcopula[:, params_list['order'][copula]] -
+                archimedean.lsum(dcopula, is_log=True, axis=1)
+            )
     return l_state
 
 
@@ -439,8 +479,76 @@ def partial_loglikelihood_copula(u_values, copula, params_list):
     :param copula:
     :param params_list:
     :return:
+    >>> np.exp(partial_loglikelihood_copula(
+    ...    u_values = np.array([
+    ...       [0.42873569, 0.18285458, 0.9514195],
+    ...       [0.25148149, 0.05617784, 0.3378213],
+    ...       [0.79410993, 0.76175687, 0.0709562],
+    ...       [0.02694249, 0.45788802, 0.6299574],
+    ...       [0.39522060, 0.02189511, 0.6332237],
+    ...       [0.66878367, 0.38075101, 0.5185625],
+    ...       [0.90365653, 0.19654621, 0.6809525],
+    ...       [0.28607729, 0.82713755, 0.7686878],
+    ...       [0.22437343, 0.16907646, 0.5740400],
+    ...       [0.66752741, 0.69487362, 0.3329266]
+    ...    ]),
+    ...    copula = 'frank',
+    ...    params_list={
+    ...        'pi': np.log(0.8),
+    ...        'alpha': np.log([0.3333, 0.3333, 0.3333]),
+    ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
+    ...        'frank': {'theta': 6, 'pi': 0.2},
+    ...        'gumbel': {'theta': 6, 'pi': 0.2},
+    ...        'clayton': {'theta': 6, 'pi': 0.2},
+    ...        'missing': None
+    ...    }
+    ... ))
+    array([0.02917896, 0.57310548, 0.00327336, 0.02657659, 0.03474353,
+           0.50843079, 0.0142948 , 0.0400287 , 0.33422576, 0.20555988])
+    >>> u_values = np.random.rand(10000,30)
+    >>> np.count_nonzero(~np.isnan(partial_loglikelihood_copula(u_values,
+    ...    copula = 'frank',
+    ...    params_list={
+    ...        'pi': np.log(0.8),
+    ...        'alpha': np.log([0.3333, 0.3333, 0.3333]),
+    ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
+    ...        'frank': {'theta': 6},
+    ...        'gumbel': {'theta': 6},
+    ...        'clayton': {'theta': 6},
+    ...        'missing': None
+    ...    }
+    ... )))
+    10000
+    >>> u_values = np.random.rand(10000,30)
+    >>> np.count_nonzero(~np.isnan(partial_loglikelihood_copula(u_values,
+    ...    copula = 'gumbel',
+    ...    params_list={
+    ...        'pi': np.log(0.8),
+    ...        'alpha': np.log([0.3333, 0.3333, 0.3333]),
+    ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
+    ...        'frank': {'theta': 6},
+    ...        'gumbel': {'theta': 6},
+    ...        'clayton': {'theta': 6},
+    ...        'missing': None
+    ...    }
+    ... )))
+    10000
+    >>> u_values = np.random.rand(10000,30)
+    >>> np.count_nonzero(~np.isnan(partial_loglikelihood_copula(u_values,
+    ...    copula = 'clayton',
+    ...    params_list={
+    ...        'pi': np.log(0.8),
+    ...        'alpha': np.log([0.3333, 0.3333, 0.3333]),
+    ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
+    ...        'frank': {'theta': 6},
+    ...        'gumbel': {'theta': 6},
+    ...        'clayton': {'theta': 6},
+    ...        'missing': None
+    ...    }
+    ... )))
+    10000
     """
-    return params_list['alpha'][params_list['order'][copula]] * \
+    return params_list['alpha'][params_list['order'][copula]] + \
         censored_copula_density(
             u_values=u_values,
             theta=params_list[copula]['theta'],
@@ -456,7 +564,7 @@ def partial_loglikelihood_theta(u_values, params_list):
     :param params_list:
     :return:
 
-    >>> partial_loglikelihood_theta(
+    >>> np.exp(partial_loglikelihood_theta(
     ...    u_values = np.array([
     ...       [0.42873569, 0.18285458, 0.9514195],
     ...       [0.25148149, 0.05617784, 0.3378213],
@@ -470,15 +578,15 @@ def partial_loglikelihood_theta(u_values, params_list):
     ...       [0.66752741, 0.69487362, 0.3329266]
     ...    ]),
     ...    params_list={
-    ...        'pi': 0.8,
-    ...        'alpha': [0.3333, 0.3333, 0.3333],
+    ...        'pi': np.log(0.8),
+    ...        'alpha': np.log([0.3333, 0.3333, 0.3333]),
     ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
     ...        'frank': {'theta': 6, 'pi': 0.2},
     ...        'gumbel': {'theta': 6, 'pi': 0.2},
     ...        'clayton': {'theta': 6, 'pi': 0.2},
     ...        'missing': None
     ...    }
-    ... )
+    ... ))
     array([[2.91789605e-02, 1.49948785e-08, 2.21307392e-05],
            [5.73105482e-01, 9.25494385e-03, 9.37790632e-07],
            [3.27335992e-03, 4.62865538e-10, 1.66678892e-11],
@@ -489,6 +597,19 @@ def partial_loglikelihood_theta(u_values, params_list):
            [4.00287013e-02, 8.29690299e-07, 2.14728677e-04],
            [3.34225756e-01, 3.41683824e-02, 1.65024776e-02],
            [2.05559880e-01, 1.13697404e-03, 1.12665660e-02]])
+    >>> u_values = np.random.rand(10000,30)
+    >>> np.count_nonzero(~np.isnan(partial_loglikelihood_theta(u_values,
+    ...    params_list={
+    ...        'pi': np.log(0.8),
+    ...        'alpha': np.log([0.3333, 0.3333, 0.3333]),
+    ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
+    ...        'frank': {'theta': 6},
+    ...        'gumbel': {'theta': 6},
+    ...        'clayton': {'theta': 6},
+    ...        'missing': None
+    ...    }
+    ... )))
+    30000
     """
     dcopula = np.zeros((u_values.shape[0], copula_number()))
     for copula in copula_names():
@@ -526,8 +647,8 @@ def loglikelihood_theta(theta, u_values, copula, params_list, dcopula=None):
     ...    ]),
     ...    copula = "frank",
     ...    params_list={
-    ...        'pi': 0.8,
-    ...        'alpha': [0.3333, 0.3333, 0.3333],
+    ...        'pi': np.log(0.8),
+    ...        'alpha': np.log([0.3333, 0.3333, 0.3333]),
     ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
     ...        'frank': {'theta': 6, 'pi': 0.2},
     ...        'gumbel': {'theta': 6, 'pi': 0.2},
@@ -535,7 +656,7 @@ def loglikelihood_theta(theta, u_values, copula, params_list, dcopula=None):
     ...        'missing': None
     ...    }
     ... )
-    1.4747355660553327
+    1.4747355660553325
     >>> loglikelihood_theta(
     ...    theta=2,
     ...    u_values = np.array([
@@ -553,8 +674,8 @@ def loglikelihood_theta(theta, u_values, copula, params_list, dcopula=None):
     ...    ]),
     ...    copula = "frank",
     ...    params_list={
-    ...        'pi': 0.8,
-    ...        'alpha': [0.3333, 0.3333, 0.3333],
+    ...        'pi': np.log(0.8),
+    ...        'alpha': np.log([0.3333, 0.3333, 0.3333]),
     ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
     ...        'frank': {'theta': 6, 'pi': 0.2},
     ...        'gumbel': {'theta': 6, 'pi': 0.2},
@@ -562,7 +683,7 @@ def loglikelihood_theta(theta, u_values, copula, params_list, dcopula=None):
     ...        'missing': 0.0
     ...    }
     ... )
-    1.4072509026421056
+    1.4072509026421054
     """
     if dcopula is None:
         dcopula = partial_loglikelihood_theta(
@@ -570,7 +691,7 @@ def loglikelihood_theta(theta, u_values, copula, params_list, dcopula=None):
             params_list=params_list
         )
     dcopula[:, params_list['order'][copula]] = (
-        params_list['alpha'][params_list['order'][copula]] *
+        params_list['alpha'][params_list['order'][copula]] +
         censored_copula_density(
             u_values=u_values,
             theta=theta,
@@ -580,9 +701,9 @@ def loglikelihood_theta(theta, u_values, copula, params_list, dcopula=None):
     )
     return -np.sum(
         np.log(
-            params_list['pi'] +
-            (1 - params_list['pi']) *
-            np.sum(dcopula, axis=1)
+            np.exp(params_list['pi']) +
+            (1 - np.exp(params_list['pi'])) *
+            archimedean.explsum(dcopula, is_log=True, axis=1)
         ),
         axis=0
     )
@@ -607,8 +728,8 @@ def local_idr(u_values, params_list, missing=None):
     ...    [0.66752741, 0.69487362, 0.3329266]
     ...    ]),
     ...    params_list={
-    ...        'pi': 0.8,
-    ...        'alpha': [0.3333, 0.3333, 0.3333],
+    ...        'pi': np.log(0.8),
+    ...        'alpha': np.log([0.3333, 0.3333, 0.3333]),
     ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
     ...        'frank': {'theta': 6, 'pi': 0.2},
     ...        'gumbel': {'theta': 6, 'pi': 0.2},
@@ -616,16 +737,18 @@ def local_idr(u_values, params_list, missing=None):
     ...        'missing': None
     ...    }
     ... )
-    array([0.79535495, 0.71654265, 0.79947661, 0.7957702 , 0.79447936,
-           0.6802532 , 0.79771887, 0.79361233, 0.74281835, 0.76658261])
+    array([0.99275263, 0.87291239, 0.99918233, 0.99339966, 0.99138885,
+           0.81964467, 0.99643828, 0.99003915, 0.91222219, 0.94832496])
     """
-    return 1 - expectation_k(u_values=u_values, params_list=params_list)
+    return 1.0 - np.exp(
+        expectation_k(u_values=u_values, params_list=params_list)
+    )
 
 
 def minimize_alpha(l_state):
     """
     compute maximization of alpha
-    >>> minimize_alpha(np.array([
+    >>> u_values = np.array([
     ...    [0.42873569, 0.18285458, 0.9514195],
     ...    [0.25148149, 0.05617784, 0.3378213],
     ...    [0.79410993, 0.76175687, 0.0709562],
@@ -636,13 +759,39 @@ def minimize_alpha(l_state):
     ...    [0.28607729, 0.82713755, 0.7686878],
     ...    [0.22437343, 0.16907646, 0.5740400],
     ...    [0.66752741, 0.69487362, 0.3329266]
-    ...    ]))
-    array([0.46469085, 0.37489573, 0.16041342])
+    ... ])
+    >>> l_state = expectation_l(u_values=u_values,
+    ...    params_list={
+    ...        'pi': np.log(0.8),
+    ...        'alpha': np.log([0.3333, 0.3333, 0.3333]),
+    ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
+    ...        'frank': {'theta': 6},
+    ...        'gumbel': {'theta': 6},
+    ...        'clayton': {'theta': 6},
+    ...        'missing': None
+    ...    }
+    ... )
+    >>> np.exp(minimize_alpha(l_state))
+    array([0.93668671, 0.02222581, 0.04108749])
+    >>> np.round(np.sum(np.exp(minimize_alpha(l_state))), decimals=4)
+    1.0
+    >>> u_values = np.random.rand(10000,30)
+    >>> l_state = expectation_l(u_values=u_values,
+    ...    params_list={
+    ...        'pi': np.log(0.8),
+    ...        'alpha': np.log([0.3333, 0.3333, 0.3333]),
+    ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
+    ...        'frank': {'theta': 6},
+    ...        'gumbel': {'theta': 6},
+    ...        'clayton': {'theta': 6},
+    ...        'missing': None
+    ...    }
+    ... )
+    >>> np.round(np.sum(np.exp(minimize_alpha(l_state))), decimals=4)
+    1.0
     """
-    alpha = np.zeros(l_state.shape[1])
-    alpha[:-1] = np.sum(l_state[:, :-1], axis=0) / float(l_state.shape[0])
-    alpha[-1] = 1.0 - np.sum(alpha[:-1])
-    return alpha
+    return archimedean.lsum(l_state, is_log=True, axis=0) - np.log(
+        float(l_state.shape[0]))
 
 
 def build_bounds(copula, eps=1e-4):
@@ -682,13 +831,15 @@ def minimize_pi(k_state):
     find theta that minimize the likelihood of the copula density
     :param k_state:
     :return:
-    >>> minimize_pi(
-    ...     k_state = np.array([0.99275263, 0.87291238, 0.99918233, 0.99339966,
-    ...                         0.99138885, 0.81964467, 0.99643828, 0.99003915,
-    ...                         0.91222219, 0.94832496]))
-    0.95163051
+    >>> np.exp(minimize_pi(
+    ...     k_state = np.log(np.array([0.99275263, 0.87291238, 0.99918233,
+    ...                         0.99339966, 0.99138885, 0.81964467,
+    ...                         0.99643828, 0.99003915,
+    ...                         0.91222219, 0.94832496]))))
+    0.9516305099999998
     """
-    return float(np.sum(k_state)) / float(len(k_state))
+    return archimedean.lsum(k_state, is_log=True, axis=0)[0] - float(
+        np.log(len(k_state)))
 
 
 def minimize_theta(u_values, copula, params_list, dcopula=None):
@@ -714,8 +865,8 @@ def minimize_theta(u_values, copula, params_list, dcopula=None):
     ...    ]),
     ...    copula="frank",
     ...    params_list={
-    ...        'pi': 0.01,
-    ...        'alpha': [0.005, 0.005, 0.99],
+    ...        'pi': np.log(0.01),
+    ...        'alpha': np.log([0.005, 0.005, 0.99]),
     ...        'order': {'frank': 2, 'gumbel': 0, 'clayton': 1},
     ...        'frank': {'theta': 6, 'pi': 0.2},
     ...        'gumbel': {'theta': 6, 'pi': 0.2},
@@ -723,7 +874,7 @@ def minimize_theta(u_values, copula, params_list, dcopula=None):
     ...        'missing': None
     ...    }
     ... )
-    5.443893056499774
+    5.4438915807028305
     >>> # data generated with theta = 5 and 10% missing
     >>> minimize_theta(np.array([
     ...    [0.2104386, 0.6909995, 0.2195247],
@@ -739,8 +890,8 @@ def minimize_theta(u_values, copula, params_list, dcopula=None):
     ...    ]),
     ...    copula="frank",
     ...    params_list={
-    ...        'pi': 0.01,
-    ...        'alpha': [0.005, 0.005, 0.99],
+    ...        'pi': np.log(0.01),
+    ...        'alpha': np.log([0.005, 0.005, 0.99]),
     ...        'order': {'frank': 2, 'gumbel': 0, 'clayton': 1},
     ...        'frank': {'theta': 6, 'pi': 0.2},
     ...        'gumbel': {'theta': 6, 'pi': 0.2},
@@ -748,7 +899,7 @@ def minimize_theta(u_values, copula, params_list, dcopula=None):
     ...        'missing': 0.0
     ...    }
     ... )
-    4.979948165192934
+    4.979949296180444
     >>> # data generated with theta = 5
     >>> minimize_theta(np.array([
     ...    [0.24078826, 0.37870827, 0.16061743],
@@ -764,8 +915,8 @@ def minimize_theta(u_values, copula, params_list, dcopula=None):
     ...    ]),
     ...    copula="gumbel",
     ...    params_list={
-    ...        'pi': 0.01,
-    ...        'alpha': [0.005, 0.005, 0.99],
+    ...        'pi': np.log(0.01),
+    ...        'alpha': np.log([0.005, 0.005, 0.99]),
     ...        'order': {'frank': 0, 'gumbel': 2, 'clayton': 1},
     ...        'frank': {'theta': 6, 'pi': 0.2},
     ...        'gumbel': {'theta': 6, 'pi': 0.2},
@@ -773,7 +924,7 @@ def minimize_theta(u_values, copula, params_list, dcopula=None):
     ...        'missing': None
     ...    }
     ... )
-    4.5616062095128775
+    4.5616032335264665
     >>> # data generated with theta = 5 and 10% missing
     >>> minimize_theta(np.array([
     ...    [0.24078826, 0.37870827, 0.16061743],
@@ -789,8 +940,8 @@ def minimize_theta(u_values, copula, params_list, dcopula=None):
     ...    ]),
     ...    copula="gumbel",
     ...    params_list={
-    ...        'pi': 0.01,
-    ...        'alpha': [0.005, 0.005, 0.99],
+    ...        'pi': np.log(0.01),
+    ...        'alpha': np.log([0.005, 0.005, 0.99]),
     ...        'order': {'frank': 0, 'gumbel': 2, 'clayton': 1},
     ...        'frank': {'theta': 6, 'pi': 0.2},
     ...        'gumbel': {'theta': 6, 'pi': 0.2},
@@ -798,7 +949,7 @@ def minimize_theta(u_values, copula, params_list, dcopula=None):
     ...        'missing': 0.0
     ...    }
     ... )
-    4.106380760237294
+    4.106380435452723
     >>> # data generated with theta = 5
     >>> minimize_theta(np.array([
     ...    [0.10112599, 0.13290793, 0.180375718],
@@ -814,8 +965,8 @@ def minimize_theta(u_values, copula, params_list, dcopula=None):
     ...    ]),
     ...    copula="clayton",
     ...    params_list={
-    ...        'pi': 0.01,
-    ...        'alpha': [0.005, 0.005, 0.99],
+    ...        'pi': np.log(0.01),
+    ...        'alpha': np.log([0.005, 0.005, 0.99]),
     ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
     ...        'frank': {'theta': 6, 'pi': 0.2},
     ...        'gumbel': {'theta': 6, 'pi': 0.2},
@@ -823,7 +974,7 @@ def minimize_theta(u_values, copula, params_list, dcopula=None):
     ...        'missing': None
     ...    }
     ... )
-    5.340785328498203
+    5.340780997857822
     >>> # data generated with theta = 5 and 10% missing data
     >>> minimize_theta(np.array([
     ...    [0.00000000, 0.13290793, 0.180375718],
@@ -839,8 +990,8 @@ def minimize_theta(u_values, copula, params_list, dcopula=None):
     ...    ]),
     ...    copula="clayton",
     ...    params_list={
-    ...        'pi': 0.01,
-    ...        'alpha': [0.005, 0.005, 0.99],
+    ...        'pi': np.log(0.01),
+    ...        'alpha': np.log([0.005, 0.005, 0.99]),
     ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
     ...        'frank': {'theta': 6, 'pi': 0.2},
     ...        'gumbel': {'theta': 6, 'pi': 0.2},
@@ -848,7 +999,7 @@ def minimize_theta(u_values, copula, params_list, dcopula=None):
     ...        'missing': 0.0
     ...    }
     ... )
-    5.677473176653246
+    5.677481753916402
     """
     log.logging.debug("%s", copula + " minimize_theta")
     old_theta = params_list[copula]['theta']
@@ -899,7 +1050,7 @@ def samic(x_score, threshold=1e-4, missing=None):
     ...                      theta_0=THETA_TEST_0,
     ...                      theta_1=THETA_TEST_1)
     >>> lidr = samic(DATA["X"], threshold=0.0001)
-    >>> (np.sum((lidr < 0.5).all() == DATA["K"]) / len(lidr)) # <= 0.11
+    >>> (np.sum((lidr < 0.5).all() == DATA["K"]) / len(lidr))  <= 0.11
     True
     """
     log.logging.info("%s", "computing idr")
@@ -914,24 +1065,20 @@ def samic(x_score, threshold=1e-4, missing=None):
     params_list = {'order': {}}
     i = 0
     for copula in copula_names():
-        if missing is None:
-            params_list[copula] = {
-                'theta': dmle_copula(copula)(u_values),
-                'theta_old': np.nan,
-            }
-        else:
-            params_list[copula] = {
-                'theta': 4.0,
-                'theta_old': np.nan,
-            }
+        params_list[copula] = {
+            'theta': 4.0,
+            'theta_old': np.nan,
+        }
         params_list['order'][copula] = i
         i += 1
     params_list['missing'] = missing
     params_list['k_state'] = np.zeros(u_values.shape[0])
     params_list['l_state'] = np.zeros((u_values.shape[0], copula_number()))
-    params_list['pi'] = 0.5
+    params_list['pi'] = np.log(0.5)
     params_list['pi_old'] = 0.0
-    params_list['alpha'] = np.repeat(1.0 / copula_number(), copula_number())
+    params_list['alpha'] = np.log(
+        np.repeat(1.0 / copula_number(), copula_number())
+    )
     params_list['alpha_old'] = params_list['alpha'][:]
     dcopula = partial_loglikelihood_theta(
         u_values=u_values,
@@ -990,26 +1137,15 @@ def log_samic(params_list):
     return str of pseudo_likelihood parameter estimate
     :param params_list:
     :return:
-    >>> log_samic(
-    ...    params_list={
-    ...        'pi': 0.8,
-    ...        'alpha': [0.3333, 0.3333, 0.3333],
-    ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
-    ...        'frank': {'theta': 6, 'pi': 0.2},
-    ...        'gumbel': {'theta': 6, 'pi': 0.2},
-    ...        'clayton': {'theta': 6, 'pi': 0.2}
-    ...    })
-    '{"pi": 0.8, \n"alpha": [0.3333, 0.3333, 0.3333], \n"clayton": \
-    {"theta": 6}, \n"frank": {"theta": 6}, \n"gumbel": {"theta": 6} }'
     """
     log_str = str('{' +
-                  '"pi": ' + str(params_list['pi'])
+                  '"pi": ' + str(np.exp(params_list['pi']))
                   )
     for copula in copula_names():
         log_str += str(
             ', \n"' + copula + '": {' +
             '"alpha": ' +
-            str(params_list['alpha'][params_list["order"][copula]]) +
+            str(np.exp(params_list['alpha'][params_list["order"][copula]])) +
             ', ' '"theta": ' +
             str(params_list[copula]['theta']) +
             '}')
